package com.qmth.teachcloud.report.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.report.business.bean.result.*;
import com.qmth.teachcloud.report.business.entity.TAExamCourse;
import com.qmth.teachcloud.report.business.entity.TAExamCourseRecord;
import com.qmth.teachcloud.report.business.enums.PublishStatusEnum;
import com.qmth.teachcloud.report.business.enums.SemesterEnum;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseMapper;
import com.qmth.teachcloud.report.business.mapper.TAExamCourseRecordMapper;
import com.qmth.teachcloud.report.business.service.*;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * <p>
 * 考试课程维度分析表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-06-01
 */
@Service
public class TAExamCourseServiceImpl extends ServiceImpl<TAExamCourseMapper, TAExamCourse> implements TAExamCourseService {

    @Resource
    TAExamCourseMapper taExamCourseMapper;

    @Resource
    TAExamCourseRecordMapper taExamCourseRecordMapper;

    @Resource
    TBPaperService tbPaperService;

    @Resource
    TAExamCourseCollegeInspectService taExamCourseCollegeInspectService;

    @Resource
    TBDimensionService tbDimensionService;

    @Resource
    TBCommonLevelConfigService tbCommonLevelConfigService;

    @Resource
    TBPaperStructService tbPaperStructService;

    @Resource
    TBAnswerService tbAnswerService;

    @Resource
    TAExamCourseCollegeTeacherService taExamCourseCollegeTeacherService;

    @Resource
    ReportCommonService reportCommonService;

    @Resource
    TAExamCourseRecordService taExamCourseRecordService;

    /**
     * 开课课程考试总览列表接口
     *
     * @param iPage
     * @param examId
     * @param semester
     * @param schoolId
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyTeacherList(IPage<Map> iPage, Long examId, SemesterEnum semester, Long schoolId) {
        return taExamCourseMapper.surveyTeacherList(iPage, examId, semester.name(), schoolId);
    }


    @Override
    public TAExamCourse getOverview(String semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return taExamCourseMapper.getOverview(schoolId, semester, examId, courseCode);
    }

    /**
     * 赋分管理列表接口
     *
     * @param iPage
     * @param schoolId
     * @param examId
     * @param courseCode
     * @param publishStatus
     * @return
     */
    @Override
    public IPage<TAExamCourseResult> surveyAspointsList(IPage<Map> iPage, Long schoolId, Long examId, String courseCode, PublishStatusEnum publishStatus) {
        return taExamCourseMapper.surveyAspointsList(iPage, schoolId, examId, courseCode, publishStatus.name());
    }

    @Override
    public Map<String, Object> getGradeDistribute(String semester, Long examId, String courseCode) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Map<String, Object> map = new HashMap<>();
        // 不及格率
        Map<String, String> scoreAndRate = taExamCourseMapper.getScoreAndRate(schoolId, semester, examId, courseCode);
        map.put("scoreAndRate", scoreAndRate);
        // 成绩正态分布
        String[] strings = {"90~100", "80~89", "70~79", "60~69", "55~59", "1~54"};
        List<Map<String, Object>> mapList = new ArrayList<>();
        for (String string : strings) {
            String[] str = string.split("~");
            double startScore = Double.parseDouble(str[0]);
            double endScore = Double.parseDouble(str[1]);
            List<TAExamCourseRecord> taExamCourseRecords = taExamCourseRecordMapper.listExamCourseRecord(schoolId, semester, examId, courseCode);
            // 全部有效人数
            int totalRealityCount = taExamCourseRecords.size();
            // 本分数段人数
            long totalScoreCount = taExamCourseRecords.stream().filter(m -> m.getAssignedScore().doubleValue() >= startScore && m.getAssignedScore().doubleValue() <= endScore).count();
            // 应届有效人数
            long totalCurrentCount = taExamCourseRecords.stream().filter(TAExamCourseRecord::getStudentCurrent).count();
            // 本分数段应届有效人数
            long totalScoreCurrentCount = taExamCourseRecords.stream().filter(m -> m.getStudentCurrent() && m.getAssignedScore().doubleValue() >= startScore && m.getAssignedScore().doubleValue() <= endScore).count();
            Map<String, Object> objectMap = new HashMap<>();
            objectMap.put("scores", String.join(",", str));
            objectMap.put("totalRealityCount", totalRealityCount);
            objectMap.put("totalRate", totalRealityCount == 0 ? 0 : new BigDecimal(totalScoreCount).divide(new BigDecimal(totalRealityCount), 2, BigDecimal.ROUND_HALF_UP));
            objectMap.put("currentRealityCount", totalCurrentCount);
            objectMap.put("totalRate", totalCurrentCount == 0 ? 0 : new BigDecimal(totalScoreCurrentCount).divide(new BigDecimal(totalCurrentCount), 2, BigDecimal.ROUND_HALF_UP));
            mapList.add(objectMap);
        }
        map.put("grades", mapList);
        return map;
    }

    /**
     * 考查课程考试分析接口
     *
     * @param schoolId
     * @param semester
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.COLLEGE_COURSE_REPORT, key = "#schoolId + '-' + #semester + '-' + #examId + '-' + #courseCode")
    public CollegeAndCourseResult surveyInspectAnalyseView(Long schoolId, SemesterEnum semester, Long examId, String courseCode) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        if (Objects.isNull(sysUser)) {
            throw ExceptionResultEnum.NOT_LOGIN.exception();
        }
        //学院学科报表查询科目信息
        CourseInfoResult courseInfoResult = reportCommonService.findCourseInfo(examId, courseCode, sysUser.getOrgId());
        //学院学科报表科目描述
        CollegeAndSchoolGradeDistributionResult collegeAndSchoolGradeDistributionResult = reportCommonService.findCourseDistribution(examId, courseCode, sysUser.getOrgId());
        //学院学科报表维度查询
        DimensionAnalyzeResult dimensionAnalyzeResult = reportCommonService.findDimensionInfo(examId, courseCode, sysUser.getOrgId());
        //查找试卷结构
        List<TBPaperStructResult> questionDatasource = tbPaperStructService.findQuestionInfo(examId, courseCode);
        //查找题目相关
        QuestionInfoResult questionInfoResult = reportCommonService.findSituationOfQuestions(examId, courseCode, sysUser.getOrgId(), questionDatasource);
        //查找老师得分
        TeacherInfoListResult teacherInfoListResult = reportCommonService.findTeacherInfo(examId, courseCode, sysUser.getOrgId());
        //查找题目信息
        QuestionListResult questionListResult = reportCommonService.findQuestionInfo(examId, courseCode, sysUser.getOrgId(), questionDatasource);

        return new CollegeAndCourseResult(courseInfoResult, collegeAndSchoolGradeDistributionResult, dimensionAnalyzeResult,
                questionInfoResult, teacherInfoListResult, questionListResult);
    }
}
